// CheckListBox.cpp: implementation of the CCheckListBox class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CheckListBox.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCheckListBox::CCheckListBox(): m_CheckMessage(0)
{

}

CCheckListBox::~CCheckListBox()
{

}


void CCheckListBox::DrawItem(LPDRAWITEMSTRUCT pDIS)
{
	COLORREF OldTextColor;
	COLORREF OldBkColor;
	COLORREF TextColot;
	COLORREF BackColor;
	char str[256];

	if (pDIS->itemID == -1)
	{
		return;
	}

	int len = GetText(pDIS->itemID, str);

	if (pDIS->itemState & ODS_SELECTED)
	{
		TextColot = RGB(255, 255, 255);
		BackColor = RGB(0, 0, 255);
	}
	else
	{
		BackColor = RGB(255, 255, 255);
		TextColot = RGB(0, 0, 0);
	}
	
	RECT rect = pDIS->rcItem;
	rect.left += 24;

	CBrush brush;
	HBRUSH OldBrush;
	brush.CreateSolidBrush(BackColor);

	CPen pen;
	HPEN OldPen;
	pen.CreatePen(PS_SOLID, 1, BackColor);

	OldPen = (HPEN)SelectObject(pDIS->hDC, pen);
	OldBrush = (HBRUSH)SelectObject(pDIS->hDC, brush);
	OldTextColor = SetTextColor(pDIS->hDC, TextColot);
	OldBkColor = SetBkColor(pDIS->hDC, BackColor);

	Rectangle(pDIS->hDC, pDIS->rcItem.left, pDIS->rcItem.top, 
		pDIS->rcItem.right, pDIS->rcItem.bottom);
	DrawText(pDIS->hDC, str, len, &rect, DT_LEFT | DT_WORD_ELLIPSIS);

	SetBkColor(pDIS->hDC, OldBkColor);
	SetTextColor(pDIS->hDC, OldTextColor);
	SelectObject(pDIS->hDC, OldBrush);
	SelectObject(pDIS->hDC, OldPen);

	RECT FrameRect = pDIS->rcItem;
	FrameRect.left += LEFTBORDERCHECK;
	FrameRect.right = 18;

	if (m_Flags[pDIS->itemID])
	{
		DrawFrameControl(pDIS->hDC, &FrameRect, DFC_BUTTON, DFCS_BUTTONCHECK | DFCS_CHECKED);
	}
	else
	{
		DrawFrameControl(pDIS->hDC, &FrameRect, DFC_BUTTON, DFCS_BUTTONCHECK);
	}
}



BOOL CCheckListBox::OnCreate (LPCREATESTRUCT lpCreateStruct)
{
	return this->DefWindowProc(WM_CREATE, 0, (LPARAM)lpCreateStruct);
}

void CCheckListBox::OnLButtonDown (UINT key, CPoint point)
{
	BOOL out = FALSE;
	unsigned int CurrItem = this->ItemFromPoint(point, out);
	unsigned int Size = this->GetCount();

	if (point.x >= LEFTBORDERCHECK && point.x <= (LEFTBORDERCHECK + 16)
		&& CurrItem >= 0 && CurrItem < Size)
	{
		ATLASSERT (m_Flags.size() == Size);
		m_Flags[CurrItem] = !m_Flags[CurrItem];

		RECT ItemRect;
		this->GetItemRect(CurrItem, &ItemRect);
		InvalidateRect (&ItemRect);

		::SendMessage(this->GetParent(), m_CheckMessage, WPARAM(CurrItem), LPARAM(m_Flags[CurrItem]));
	}
	
	this->DefWindowProc(WM_LBUTTONDOWN, key, MAKELPARAM(point.x, point.y));
}

void CCheckListBox::AddString (const char *str, BOOL Check)
{
	m_Flags.push_back(Check);
	CWindowImpl<CCheckListBox, CListBox, CheckStyles>::AddString(str);
	ATLASSERT (this->GetCount() == m_Flags.size());
}

void CCheckListBox::DeleteString (int Number)
{
	int Count = this->GetCount();
	
	ATLASSERT (Number >= 0 && Number < Count);
	m_Flags.erase(m_Flags.begin() + Number);
	CWindowImpl<CCheckListBox, CListBox, CheckStyles>::DeleteString(Number);
}

BOOL CCheckListBox::IsCheck(int Number)
{
	ATLASSERT (Number >= 0 && Number < m_Flags.size());
	return m_Flags[Number];
}

HWND CCheckListBox::Create(HWND hWndParent, DWORD Message, _U_RECT rect,
		LPCTSTR szWindowName, DWORD dwStyle, DWORD dwExStyle,
		_U_MENUorID MenuOrID)
{
	m_CheckMessage = Message;

	return CWindowImpl<CCheckListBox, CListBox, CheckStyles>::Create(hWndParent, 
		*(rect.m_lpRect), NULL, dwStyle, dwExStyle, (UINT)MenuOrID.m_hMenu, NULL);
}

void CCheckListBox::Clear()
{
	int Count = this->GetCount();
	for (int i = 0; i < Count; ++i)
	{
		DeleteString(0);
	}
}